(* :Title: Basic Regression *)

(* :Context: RMCPackages`BasicRegression` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to fit linear, exponential,
and power functions to a set of bivariate numerical data
using the method of least squares. Graphical displays of
the functions and residuals are available.
*)

(* :Package Version: 2.5 *)

(* :Mathematica Version: 5.0 *)

(* :History:
	V1.0 by Bruce F. Torrence, 1995.
	V2.0 by Bruce F. Torrence, 1998.
	V2.1 by Bruce F. Torrence, Aug 2000.
	V2.5 by Bruce F. Torrence, Dec 2003. 
*)

(* :Keywords: regression, bivariate, data analysis *)


BeginPackage["RMCPackages`BasicRegression`"]

ScatterPlot::usage = "ScatterPlot[xdata, ydata] produces a 
scatterplot of the data whose x coordinates are stored in the
list xdata and whose y coordinates are stored in the list
ydata. ScatterPlot[data] will also produce such a plot, where 
data is a list of x-y pairs of the form {{x1,y1},{x2,y2},...}.
ScatterPlot can accept any options that ListPlot accepts."

LinearFit::usage = "LinearFit[xdata, ydata] gives the intercept
and slope of the least-squares regression line, the value of r-squared,
and a scatterplot (with the regression line superimposed) of 
the data whose x coordinates are stored in the
list xdata and whose y coordinates are stored in the list
ydata. LinearFit[data] will also produce this output, where 
data is a list of x-y pairs of the form {{x1,y1},{x2,y2},...}.
LinearFit can accept any options accepted by ListPlot." 

PowerFit::usage = "PowerFit[xdata, ydata] gives the equation
of the best fitting power function to the data whose x coordinates 
are stored in the list xdata and whose y coordinates are stored 
in the list ydata. This function is obtained by performing
linear regression on Log[10,ydata] vs. Log[10,xdata]. Also 
reported are the value of r-squared, and a scatterplot of the data 
with the power function superimposed. PowerFit[data] will also produce 
this output, where data is a list of x-y pairs of the form 
{{x1,y1},{x2,y2},...}. PowerFit can accept any options accepted by ListPlot."

ExponentialFit::usage = "ExponentialFit[xdata, ydata] gives the 
equation of the best fitting exponential function to the data 
whose x coordinates are stored in the list xdata and whose y 
coordinates are stored in the list ydata. This function is obtained 
by performing linear regression on Log[10,ydata] vs. xdata. Also 
reported are the value of r-squared, and a scatterplot of the data 
with the exponential function superimposed. ExponentialFit[data] 
will also produce this output, where data is a list of x-y pairs 
of the form {{x1,y1},{x2,y2},...}. ExponentialFit can accept any 
options accepted by ListPlot."

SquaredResidualPlot::usage = "SquaredResidualPlot[xdata,ydata,
y-intercept,slope] produces a scatterplot of the points whose x
coordinates are stored in the list xdata and whose y coordinates 
are stored in the list ydata, together with a graph of the line
whose y-intercept and slope are specified. The scatterplot will
superimpose a square (one for each data point) whose area represents
the square of that data point's residual. Thus the picture provides
a geometrical interpretation of the sum of the squares of residuals.
If the y-intercept and slope are omitted, the least-squares 
regression line will be used. Also, the first two arguments xdata
and ydata may be replaced with a list of 2-tuples of the form
{{x1,y1},{x2,y2},...}. SquaredResidualPlot will accept any options
accepted by ListPlot."

ResidualTable::usage = "ResidualTable[xdata,ydata] gives a table
of values for the data whose x coordinates are stored in the list 
xdata and whose y coordinates are stored in the list ydata. Also 
reported are the predicted responses according to
the least-squares regression line, the residuals, and the squares
of the residuals. ResidualTable[data] will also produce this
output, where data is a list of x-y pairs of the form 
{{x1,y1},{x2,y2},...}.

ResidualTable[xdata,ydata,intercept,slope] or 
ResidualTable[data,intercept,slope]
gives a similar table, but bases the predicted responses on
the line with the given y-intercept and slope. The sum of squares of \
residuals
is reported in either case, as is a plot of the data with the
appropriate line superimposed, and a residual plot. ResidualTable
can accept any of the options accepted by ListPlot."

LinearResidualPlot::usage = "LinearResidualPlot[xdata, ydata] gives a 
residual plot for the best fitting linear function to the data whose 
x coordinates are stored in the list xdata and whose y coordinates 
are stored in the list ydata. LinearResidualPlot[data] will also 
produce this output, where data is a list of x-y pairs of the form 
{{x1,y1},{x2,y2},...}. LinearResidualPlot can accept any of the 
options accepted by ListPlot."

ExponentialResidualPlot::usage = "ExponentialResidualPlot[xdata, ydata] 
gives a residual plot for the best fitting exponential function to the data \
whose 
x coordinates are stored in the list xdata and whose y coordinates 
are stored in the list ydata. ExponentialResidualPlot[data] will also 
produce this output, where data is a list of x-y pairs of the form 
{{x1,y1},{x2,y2},...}. ExponentialResidualPlot can accept any of the 
options accepted by ListPlot."

PowerResidualPlot::usage = "PowerResidualPlot[xdata, ydata] 
gives a residual plot for the best fitting power function to the data whose 
x coordinates are stored in the list xdata and whose y coordinates 
are stored in the list ydata. PowerResidualPlot[data] will also 
produce this output, where data is a list of x-y pairs of the form 
{{x1,y1},{x2,y2},...}. PowerResidualPlot can accept any of the 
options accepted by ListPlot."

Begin["`Private`"]

BasicRegression::badarg1 = "Oops, your input to `1` needs to be
either a list of bivariate data of the form {{x1,y1},{x2,y2},...}, 
or a pair of lists of the form {x1,x2,...},{y1,y2,...}.";

BasicRegression::badarg2 = "Oops, your xdata list
is not the same length as your ydata list (or else there is only
one data point). Check your data entry for a typo.";

BasicRegression::badarg3 = "No can do. Data contains 
negative numbers or zeros.";

BasicRegression::badarg4 = "No can do. Response Variable contains 
negative numbers or zeros.";

Lo[vec_List]:=1.1 Min[vec] - .1 Max[vec] 
(* a little smaller than the min of the list vec *)

Hi[vec_List]:=1.1 Max[vec] - .1 Min[vec]
(* a little larger than the max of the list vec *)

NonPos[vec_List]:=Apply[Or,Flatten[NonPositive[vec]]]
(* returns True if any member of vec is zero or negative *)

rSquared[x_List,y_List]:=Module[{vx=Variance[x],vy=Variance[y]},
    If[vx=!=0&&vy=!=0,(Length[x]-1)^(-2)*
	vx^(-1)*vy^(-1)*((x-Mean[N[x]]).(y-Mean[N[y]]))^2,
	"Indeterminate"]
]

PerformScatterPlot[xdata_List, ydata_List, opts___Rule]:=
 Module[{data},
  data=Transpose[{xdata,ydata}];
  ListPlot[data, opts,
	AxesOrigin->{Lo[xdata],Lo[ydata]},
    PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}}, 
    Prolog->AbsolutePointSize[3],
    DefaultFont->{"Times",10}]
]

PerformScatterPlot[data_List, opts___Rule]:=
 Module[{xdata, ydata},
  xdata=Transpose[data][[1]];
  ydata=Transpose[data][[2]];
  ListPlot[data, opts,
	AxesOrigin->{Lo[xdata],Lo[ydata]},
    PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}}, 
    Prolog->AbsolutePointSize[3],
    DefaultFont->{"Times",10}]
]

ScatterPlot[xdata_List,ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  PerformScatterPlot[xdata,ydata,opts]
 ]

ScatterPlot[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, ScatterPlot],
  PerformScatterPlot[data,opts]
 ]

PerformLinearFit[xdata_List,ydata_List, opts___Rule]:=
 Module[{data,line,int,l,p},
  data=Transpose[{xdata,ydata}];
  line=Fit[data,{1,x},x];
  int=Chop[line /. x->0];     
  Print["intercept = ", int];
  Print["slope = ", Chop[line - int /. x->1]];
  Print[r^2 /. r->"r"," = ",PaddedForm[rSquared[xdata,ydata],{3,3}]];
  l=ListPlot[data, opts,
		AxesOrigin->{Lo[xdata],Lo[ydata]}, 
        PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}},
		Prolog->AbsolutePointSize[3],
        DisplayFunction->Identity];
  p=Plot[line, {x,Lo[xdata],Hi[xdata]},
        PlotStyle->{Hue[.7]},
        DisplayFunction->Identity];
  Show[l,p,DisplayFunction->$DisplayFunction]
 ]

PerformLinearFit[data_List, opts___Rule]:=
 Module[{tdata=Transpose[data]},
  PerformLinearFit[tdata[[1]],tdata[[2]],opts]
 ]
 
LinearFit[xdata_List,ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata]||Length[xdata]<2,
  Message[BasicRegression::badarg2],
  PerformLinearFit[xdata,ydata,opts]
 ]

LinearFit[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2}||Length[data]<2,
  Message[BasicRegression::badarg1, LinearFit],
  PerformLinearFit[data,opts]
 ]

PerformPowerFit[xdata_List,ydata_List, opts___Rule]:=
 Module[{data,line,k,n,l,p},
    data=Transpose[{Log[10,xdata],Log[10,ydata]}];
    line=Fit[data,{1,x},x];  
    k=line /. x->0;
    n=line - k /. x->1;
    Print["y = ", 10^k x^n /. x->"x"];
    Print[r^2 /. r->"r"," = ",
    	PaddedForm[rSquared[data[[All,1]],data[[All,2]]],{3,3}]];
    l=ListPlot[Transpose[{xdata,ydata}], opts,
         AxesOrigin->{Lo[xdata],Lo[ydata]},
		 PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}}, 	 
         Prolog->AbsolutePointSize[3],
         DisplayFunction->Identity];
    p=Plot[10^k x^n, {x,Max[{Lo[xdata],0}],Hi[xdata]},
         PlotStyle->{Hue[.7]},
         DisplayFunction->Identity];
    Show[l,p,DisplayFunction->$DisplayFunction]
 ]

PerformPowerFit[data_List, opts___Rule]:=
 Module[{tdata=Transpose[data]},
    PerformPowerFit[tdata[[1]],tdata[[2]],opts]
 ]

PowerFit[xdata_List,ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata]||Length[xdata]<2,
  Message[BasicRegression::badarg2],
  If[NonPos[xdata] || NonPos[ydata],
    Message[BasicRegression::badarg3],
    PerformPowerFit[xdata,ydata,opts]
  ]
] 

PowerFit[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2}||Length[data]<2,
  Message[BasicRegression::badarg1, PowerFit],
  If[NonPos[data],
    Message[BasicRegression::badarg3],
    PerformPowerFit[data,opts]
  ]
] 

PerformExponentialFit[xdata_List,ydata_List, opts___Rule]:=
 Module[{data,line,k,n,l,p},
  data=Transpose[{xdata,Log[10,ydata]}];
  line=Fit[data,{1,x},x];  
  k=line /. x->0;
  n=line - k /. x->1;
  Print["y = ", 10^k, "  ", (10^n)^x /. x->"x"];
  Print[r^2 /. r->"r"," = ",PaddedForm[rSquared[xdata,data[[All,2]]],{3,3}]];
  l=ListPlot[Transpose[{xdata,ydata}], opts,
         AxesOrigin->{Lo[xdata],Lo[ydata]},
		 PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}}, 
         Prolog->AbsolutePointSize[3],
         DisplayFunction->Identity];
  p=Plot[10^k (10^n)^x, {x,Lo[xdata],Hi[xdata]},
  	     PlotStyle->{Hue[.7]},
         DisplayFunction->Identity];
  Show[l,p,DisplayFunction->$DisplayFunction]
 ]
 
PerformExponentialFit[data_List, opts___Rule]:=
 Module[{tdata=Transpose[data]},
    PerformExponentialFit[tdata[[1]],tdata[[2]],opts]
 ]

ExponentialFit[xdata_List,ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata]||Length[xdata]<2,
  Message[BasicRegression::badarg2],
  If[NonPos[ydata],
    Message[BasicRegression::badarg4],
    PerformExponentialFit[xdata,ydata,opts]
  ]
]   

ExponentialFit[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2}||Length[data]<2,
  Message[BasicRegression::badarg1, ExponentialFit],
  If[NonPos[Transpose[data][[2]]],
    Message[BasicRegression::badarg4],
    PerformExponentialFit[data,opts]
  ]
] 

PerformSquaredResidualPlot[xdata_List, ydata_List, int_?NumberQ, 
slope_?NumberQ, opts___Rule]:=
 Module[{x,data,line,pred,resid,xrange,yrange,i,l,p,squares,segs},
  data=Transpose[{xdata,ydata}];
  line=int + slope*x;    
  If[slope==0,pred=Table[int,{i,Length[xdata]}],
   pred=line /. x->xdata];
  resid=If[slope>0, ydata-pred, pred-ydata]; 
  xrange=Join[xdata-resid,xdata+resid];
  yrange=Join[ydata,pred];
  l=ListPlot[data, opts,
		   AxesOrigin->{Lo[xrange],Lo[yrange]}, 
        PlotRange->{{Lo[xrange],Hi[xrange]},{Lo[yrange],Hi[yrange]}},
		Prolog->AbsolutePointSize[3],
        DisplayFunction->Identity];
  p=Plot[line, {x,Lo[xdata],Hi[xdata]},opts,
        PlotStyle->{Hue[.7]},
        DisplayFunction->Identity];
  squares=Insert[
        MapThread[
          Line[{{#1,#3},{#1-#4,#3},{#1-#4,#2},{#1,#2}}]&,{xdata,ydata,
            pred,resid}],
	    Hue[1],1];     
  segs=Insert[
        MapThread[Line[{{#1,#2},{#1,#3}}]&,{xdata,ydata,pred}],
	   Dashing[{.01}],1];     
  Show[l,p,Graphics[squares],Graphics[segs],
      DisplayFunction->$DisplayFunction,AspectRatio->Automatic,
        AxesLabel->{"x", "y"}];
]

PerformSquaredResidualPlot[data_List, int_?NumberQ, slope_?NumberQ,
  opts___Rule]:=
 Module[{tdata=Transpose[data]},
 PerformSquaredResidualPlot[tdata[[1]],tdata[[2]],int,slope,opts]
 ]
 
PerformSquaredResidualPlot[xdata_List, ydata_List, opts___Rule]:=
 Module[{data,line,x,int,slope},
  data=Transpose[{xdata,ydata}];
  line=Fit[data,{1,x},x];  
  int=line /. x->0;
  slope=line - int /. x->1;
  PerformSquaredResidualPlot[xdata,ydata,int,slope,opts]
 ]  

PerformSquaredResidualPlot[data_List,opts___Rule]:=
 Module[{line,x,int,slope},
  line=Fit[data,{1,x},x];  
  int=line /. x->0;
  slope=line - int /. x->1;
  PerformSquaredResidualPlot[data,int,slope,opts]
 ]   

SquaredResidualPlot[xdata_List,ydata_List, int_?NumberQ, 
slope_?NumberQ, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  PerformSquaredResidualPlot[xdata,ydata,int,slope,opts]
]   

SquaredResidualPlot[data_List, int_?NumberQ, slope_?NumberQ, 
opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, SquaredResidualPlot],
  PerformSquaredResidualPlot[data,int,slope,opts]
] 

SquaredResidualPlot[xdata_List, ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  PerformSquaredResidualPlot[xdata,ydata,opts]
 ]

SquaredResidualPlot[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, SquaredResidualPlot],
  PerformSquaredResidualPlot[data,opts]
 ]


PerformResidualTable[xdata_List,ydata_List,int_?NumberQ, 
slope_?NumberQ, opts___Rule]:=
 Module[{x,data,line,pred,i,resid,residsq,ssq,l,p,r,rsegs},
  data=Transpose[{xdata,ydata}];
  line=int + slope*x;    
  If[slope==0,pred=Table[int,{i,Length[xdata]}],
   pred=line /. x->xdata];
  resid=Chop[ydata - pred];
  residsq=resid^2;
  ssq=Apply[Plus,residsq];
  Print["Predicted Response = ", line /. x->"x"];
  Print["Residual = y - Predicted Response"];
  Print["Sum of squares of residuals = ", ssq];
  l=ListPlot[data, opts,
		AxesOrigin->{Lo[xdata],Lo[ydata]}, 
        PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}},
		Prolog->AbsolutePointSize[3],
        DisplayFunction->Identity];
  p=Plot[line, {x,Lo[xdata],Hi[xdata]},
        PlotStyle->{Hue[.7]},
        DisplayFunction->Identity];
  segs=Insert[
        MapThread[Line[{{#1,#2},{#1,#3}}]&,{xdata,ydata,pred}],
	    Hue[1],1];     
  Show[l,p,Graphics[segs],DisplayFunction->$DisplayFunction,
        AxesLabel->{"x", "y"}];
  r=ListPlot[Transpose[{xdata,resid}], opts,
    AxesOrigin->{Lo[xdata], 0},
    PlotRange->{{Lo[xdata],Hi[xdata]},Automatic},
    Prolog->{AbsolutePointSize[3]},
    DisplayFunction->Identity];
  rsegs=Insert[
          MapThread[Line[{{#1,#2},{#1,#3}}]&,
           {xdata,ydata-ydata,resid}],
	      Hue[1],1];     
  Show[r,Graphics[rsegs],DisplayFunction->$DisplayFunction,
        AxesLabel->{"x", "Residual"}]; 
  TableForm[Transpose[{N[xdata],N[ydata],pred,resid,residsq}],
   TableHeadings->
   {None,{"x","y","Predicted Response","Residual","Residual"^2}}]       
 ]

PerformResidualTable[xdata_List,ydata_List,opts___Rule]:=
 Module[{data,line,x,int,slope},
  data=Transpose[{xdata,ydata}];
  line=Fit[data,{1,x},x];  
  int=line /. x->0;
  slope=line - int /. x->1;
  PerformResidualTable[xdata,ydata,int,slope,opts]
 ]  

PerformResidualTable[data_List,int_?NumberQ, 
slope_?NumberQ, opts___Rule]:=
 Module[{xdata,ydata},
  xdata=Transpose[data][[1]];  
  ydata=Transpose[data][[2]];  
  PerformResidualTable[xdata,ydata,int,slope,opts]
 ]  

PerformResidualTable[data_List,opts___Rule]:=
 Module[{line,x,int,slope},
  line=Fit[data,{1,x},x];  
  int=line /. x->0;
  slope=line - int /. x->1;
  PerformResidualTable[data,int,slope,opts]
 ]  

ResidualTable[xdata_List,ydata_List, int_?NumberQ,
slope_?NumberQ, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  PerformResidualTable[xdata,ydata,int,slope,opts]
 ]

ResidualTable[xdata_List,ydata_List,opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  PerformResidualTable[xdata,ydata,opts]
 ]

ResidualTable[data_List,int_?NumberQ, slope_?NumberQ, opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, ResidualTable],
  PerformResidualTable[data,int,slope,opts]
 ]

ResidualTable[data_List,opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, ResidualTable],
  PerformResidualTable[data,opts]
 ]

LinearResidualPlot[xdata_List,ydata_List,opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  PerformLRP[xdata,ydata,opts]
 ]

LinearResidualPlot[data_List,opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, LinearResidualPlot],
  PerformLRP[data,opts]
 ]

PerformLRP[xdata_List,ydata_List, opts___Rule]:=
 Module[{data,line,pred,resid,r,rsegs},
  data=Transpose[{xdata,ydata}];
  line=Fit[data,{1,x},x];    
  pred=line /. x->xdata;
  resid=Chop[ydata - pred];
  r=ListPlot[Transpose[{xdata,resid}], opts,
    AxesOrigin->{Lo[xdata], 0},
    PlotRange->{{Lo[xdata],Hi[xdata]},Automatic},
    Prolog->{AbsolutePointSize[3]},
    DisplayFunction->Identity
  ];
 rsegs=Insert[
          MapThread[Line[{{#1,#2},{#1,#3}}]&,
           {xdata,ydata-ydata,resid}],
	      Hue[1],1];     
  Show[r,Graphics[rsegs],DisplayFunction->$DisplayFunction,
        AxesLabel->{"x", "Residual"}]; 
 ]
 
PerformLRP[data_List, opts___Rule]:=
 Module[{xdata,ydata},
  xdata=Transpose[data][[1]];
  ydata=Transpose[data][[2]];
  PerformLRP[xdata,ydata,opts]
 ]

ExponentialResidualPlot[xdata_List,ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  If[NonPos[ydata],
    Message[BasicRegression::badarg4],
    PerformERP[xdata,ydata,opts]
  ]
]   

ExponentialResidualPlot[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, ExponentialResidualPlot],
  If[NonPos[Transpose[data][[2]]],
    Message[BasicRegression::badarg4],
    PerformERP[data,opts]
  ]
] 

PerformERP[xdata_List, ydata_List, opts___Rule] := 
Module[{data, line, int, slope, resid, r, rsegs}, 
  data = Transpose[{xdata, Log[10, ydata]}]; 
  line = Fit[data, {1, x}, x]; 
  int = line /. x -> 0; 
  slope = line - int /. x -> 1; 
  resid = Chop[ydata - 10^int*10^(slope*xdata)]; 
  r=ListPlot[Transpose[{xdata,resid}], opts,
    AxesOrigin->{Lo[xdata], 0},
    PlotRange->{{Lo[xdata],Hi[xdata]},Automatic},
    Prolog->{AbsolutePointSize[3]},
    DisplayFunction->Identity
  ];
 rsegs=Insert[
          MapThread[Line[{{#1,#2},{#1,#3}}]&,
           {xdata,ydata-ydata,resid}],
	      Hue[1],1];     
  Show[r,Graphics[rsegs],DisplayFunction->$DisplayFunction,
        DefaultFont->{"Times",10},
        AxesLabel->{"x", "Residual"}];
]

PerformERP[data_List, opts___Rule]:=
 Module[{tdata=Transpose[data]},
  PerformERP[tdata[[1]],tdata[[2]],opts]
 ]

PowerResidualPlot[xdata_List,ydata_List, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
  Message[BasicRegression::badarg2],
  If[NonPos[xdata] || NonPos[ydata],
    Message[BasicRegression::badarg3],
    PerformPRP[xdata,ydata,opts]
  ]
] 

PowerResidualPlot[data_List, opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[BasicRegression::badarg1, PowerResidualPlot],
  If[NonPos[data],
    Message[BasicRegression::badarg3],
    PerformPRP[data,opts]
  ]
] 
  
PerformPRP[xdata_List, ydata_List, opts___Rule] := 
Module[{data, line, int, slope, resid, r, rsegs}, 
  data = Transpose[{Log[10,xdata], Log[10,ydata]}]; 
  line = Fit[data, {1, x}, x]; 
  int = line /. x -> 0; 
  slope = line - int /. x -> 1; 
  resid = Chop[ydata - 10^int*xdata^slope]; 
  r=ListPlot[Transpose[{xdata,resid}], opts,
    AxesOrigin->{Lo[xdata], 0},
    PlotRange->{{Lo[xdata],Hi[xdata]},Automatic},
    Prolog->{AbsolutePointSize[3]},
    DisplayFunction->Identity
  ];
 rsegs=Insert[
          MapThread[Line[{{#1,#2},{#1,#3}}]&,
           {xdata,ydata-ydata,resid}],
	      Hue[1],1];     
  Show[r,Graphics[rsegs],DisplayFunction->$DisplayFunction,
        AxesLabel->{"x", "Residual"}];
]  

PerformPRP[data_List, opts___Rule]:=
 Module[{tdata=Transpose[data]},
  PerformPRP[tdata[[1]],tdata[[2]],opts]
 ]

End[]

EndPackage[]
