(* :Title: ComplexMap *)

(* :Context: RMCPackages`ComplexMap` *)

(* :Author: Bruce Torrence, original concept Roman Maeder *)

(* :Summary: A means of graphing functions of a single complex variable *)

(* :Package Version: 1.0 *)

(* :Mathematica Version: 4.0 *)

(* :History: 1.0 for Mathematica in Education Workshop, Spring 2002, Utrecht, NL *)

(* :Keywords: CartesianMap, CartesianMapMovie *)



BeginPackage["RMCPackages`ComplexMap`", "Utilities`FilterOptions`"]

CartesianMap::usage = "CartesianMap[f,{xmin,xmax},{ymin,ymax}] produces a pair of graphics.
The left shows a grid in the complex plane covering the region from xmin to xmax, and ymin
to ymax. The right graphic shows the images under f of the intersection points of the
grid, with these points joined by line segments. The horizontal and vertical grid lines
(and their images) can be colored with the options XLineStyle and YLineStyle. Set these
options to a graphics directive (or to a list of graphics directives). The number of grid
lines (20 in each direction by default) can be adjusted in two ways: set the option
GridSize to a positive integer (or to a list of two positive integers if you want different
numbers of horizontal and vertical lines); or change the iterators to {xmin, xmax, dx} and/or
{ymin,ymax,dy}. CartesianMap will also accept any options accepted by Graphics."

CartesianMapMovie::usage = "CartesianMapMovie[f,{xmin,xmax},{ymin,ymax}] produces an
animation based on CartesianMap. The option Progression may be set to any one of LR,
RL, BT, or TB. These stand for (Left to Right), (Right to Left), (Bottom to Top), and 
(Top to Bottom), respectively, and control the direction in which the red line segment
moves. Other options include the options of CartesianMap, and any options accepted by
Graphics."

GridSize::usage = "GridSize is an option for CartesianMap and CartesianMapMovie. Set it
to a positive integer (or to a list of two positive integers if you want a different
number of horizontal and vertical grid lines)."

XLineStyle::usage  = "XLineStyle is an option for CartesianMap and CartesianMapMovie. Set it
to a graphics directive (or a list of graphics directives) to control appearance of
the vertical grid lines and their images."

YLineStyle::usage  = "YLineStyle is an option for CartesianMap and CartesianMapMovie. Set it
to a graphics directive (or a list of graphics directives) to control appearance of
the horizontal grid lines and their images."

Progression::usage = "Progression is an option for CartesianMapMovie. Set it
to any one of LR, RL, BT, or TB. These stand for (Left to Right), (Right to Left), (Bottom to Top), 
and (Top to Bottom), respectively, to control the direction in which the red line segment
moves."

LR::usage = "LR is a setting for the option Progression when using the command CartesianMapMovie."

RL::usage = "RL is a setting for the option Progression when using the command CartesianMapMovie."

BT::usage = "BT is a setting for the option Progression when using the command CartesianMapMovie."

TB::usage = "TB is a setting for the option Progression when using the command CartesianMapMovie."

CartesianMap::badGridSize = "GridSize must be set to a positive integer, or to
a list of two positive integers."

CartesianMapMovie::badProgression = "Progression must be set to one of the four
values LR, RL, BT, or TB. These stand for (Left to Right), (Right to Left),
(Bottom to Top), and (Top to Bottom), respectively."

Options[CartesianMap] = {
	GridSize -> 20, 
	XLineStyle -> GrayLevel[.7], 
    YLineStyle -> Hue[.1], 
    ImageSize -> 500
    }; 

Options[CartesianMapMovie] = {
    GridSize->20,
    XLineStyle->GrayLevel[.7],
    YLineStyle->GrayLevel[.7],
    Progression->LR,
    ImageSize->500
    };



Begin["`Private`"]



CartesianMap[f_, {xmin_, xmax_, dx_:Automatic}, {ymin_, ymax_, dy_:Automatic}, opts___Rule] := 
  Module[{gs, ndx = dx, ndy = dy, fpts, coords, fcoords, xls, yls, gridvlines, gridhlines, is}, 
    gs = GridSize /. {opts} /. Options[CartesianMap]; 
    gs = Switch[gs, 
    	_Integer?Positive, {gs, gs}, 
    	{_Integer?Positive, _Integer?Positive}, gs, 
    	_, Return[Message[CartesianMap::badGridSize]]
    	]; 
    If[dx === Automatic, ndx = N[(xmax - xmin)/(gs[[1]] - 1)]]; 
    If[dy === Automatic, ndy = N[(ymax - ymin)/(gs[[2]] - 1)]]; 
    pts = Table[N[x + I*y], {x, xmin, xmax, ndx}, {y, ymin, ymax, ndy}]; 
    fpts = Map[f, pts, {2}]; coords = Map[{Re[#1], Im[#1]} & , pts, {2}]; 
    fcoords = Map[{Re[#1], Im[#1]} & , fpts, {2}]; 
    xls = XLineStyle /. {opts} /. Options[CartesianMap]; 
    yls = YLineStyle /. {opts} /. Options[CartesianMap]; 
    If[Head[xls] =!= List, xls = {xls}]; 
    If[Head[yls] =!= List, yls = {yls}]; 
    vlines = Line /@ fcoords; 
    hlines = Line /@ Transpose[fcoords]; 
    gridhlines = MapThread[Line[{#1, #2}]& , {First[coords], Last[coords]}]; 
    gridvlines = MapThread[Line[{#1, #2}]& , {coords[[All,1]], coords[[All,-1]]}]; 
    is = ImageSize /. {opts} /. Options[CartesianMap]; 
    Show[GraphicsArray[
      {Graphics[{{Sequence @@ xls, gridvlines}, {Sequence @@ yls, gridhlines}}, 
        FilterOptions[Graphics, opts], AspectRatio -> Automatic, Axes -> Automatic], 
       Graphics[{{Sequence @@ xls, vlines}, {Sequence @@ yls, hlines}}, 
        FilterOptions[Graphics, opts], AspectRatio -> Automatic, Axes -> Automatic]
      }],ImageSize->is]
    ]
    

CartesianMapMovie[f_, {xmin_, xmax_, dx_:Automatic}, {ymin_, ymax_, dy_:Automatic}, opts___Rule] := 
  Module[{gs, ndx = dx, ndy = dy, pts, fpts, coords, fcoords, xls, yls, 
     gridvlines, gridhlines, leftfr, rightfr, leftframe, rightframe, i, dir, partspec, enddo, is}, 
    gs = GridSize /. {opts} /. Options[CartesianMapMovie]; 
    gs = Switch[gs, 
    	_Integer?Positive, {gs, gs}, 
    	{_Integer?Positive, _Integer?Positive}, gs, 
    	_, Return[Message[CartesianMap::badGridSize]]]; 
    If[dx === Automatic, ndx = N[(xmax - xmin)/(gs[[1]] - 1)]]; 
    If[dy === Automatic, ndy = N[(ymax - ymin)/(gs[[2]] - 1)]]; 
    pts = Table[N[x + I*y], {x, xmin, xmax, ndx}, {y, ymin, ymax, ndy}]; 
    fpts = Map[f, pts, {2}]; coords = Map[{Re[#1], Im[#1]} & , pts, {2}]; 
    fcoords = Map[{Re[#1], Im[#1]} & , fpts, {2}]; 
    xls = XLineStyle /. {opts} /. Options[CartesianMapMovie]; 
    yls = YLineStyle /. {opts} /. Options[CartesianMapMovie]; 
    If[Head[xls] =!= List, xls = {xls}]; 
    If[Head[yls] =!= List, yls = {yls}]; 
    vlines = Line /@ fcoords; 
    hlines = Line /@ Transpose[fcoords]; 
    gridhlines = MapThread[Line[{#1, #2}] & , {First[coords], Last[coords]}]; 
    gridvlines = MapThread[Line[{#1, #2}] & , {coords[[All,1]], coords[[All,-1]]}]; 
    leftfr = {{Sequence @@ xls, gridvlines}, {Sequence @@ yls, gridhlines}}; 
    rightfr = {{Sequence @@ xls, vlines}, {Sequence @@ yls, hlines}}; 
    dir = Progression /. {opts} /. Options[CartesianMapMovie]; 
    partspec[i_] = Which[
    	dir === LR, {1, 2, i}, 
    	dir === RL, {1, 2, -i}, 
      	dir === BT, {2, 2, i}, 
      	dir === TB, {2, 2, -i}, 
      	True, Return[Message[CartesianMapMovie::badProgression]]
      	]; 
    enddo = If[partspec[1][[1]] == 1, gs[[1]], gs[[2]]]; 
    is = ImageSize /. {opts} /. Options[CartesianMapMovie];
    Do[
    	leftframe = ReplacePart[leftfr, {Hue[1],Thickness[.01],leftfr[[Sequence @@ partspec[i]]]}, partspec[i]]; 
      	rightframe = ReplacePart[rightfr, {Hue[1],Thickness[.02],rightfr[[Sequence @@ partspec[i]]]}, partspec[i]]; 
      	Show[GraphicsArray[
      		{Graphics[leftframe, 
      			FilterOptions[Graphics, opts], AspectRatio -> Automatic], 
         	 Graphics[rightframe, 
         		FilterOptions[Graphics, opts], AspectRatio -> Automatic]
         	}],ImageSize->is], 
        {i, enddo}]
    ]

Protect[CartesianMap,CartesianMapMovie]

End[]

EndPackage[]
