(* :Package Version: 1.0 *)

(* :Mathematica Version: 3.0 *)

(* :Name: RMCPackages`ODETools` *)

(* :Title: Ordinary Differential Equation Tools *)

(* :Author: Bruce F. Torrence *)

(* :Keywords:
    ODE, Ordinary Differential Equation, 
    Differential Equation, DirectionField *)

(*:Summary: This package provides some basic tools
for working with ordinary differential equations
in an educational setting. *)

BeginPackage["RMCPackages`ODETools`",
	"Graphics`PlotField`", 
    "Utilities`FilterOptions`"]

DirectionField::usage =
"DirectionField[f, {x, xmin, xmax}, {y, ymin, ymax}]
produces a direction field plot of the first order differential
equation dy/dx = f[x,y]. Use the option PlotPoints to change
the number of slope lines in the output. By default PlotPoints 
is set to 11, meaning there are 11 slope lines
drawn per side, for 121 lines total. Use the setting 
ArrowHeads->True to place arrowheads on the slope lines.
Use the option ScaleArrows to change the length of the
slope lines: smaller numbers yield shorter arrows. The default
setting is 0.8. Values greater than 1 may lead to intersecting
slope lines.";

ArrowHeads::usage = "ArrowHeads is an option for
DirectionField. It is set to False by default. The
setting ArrowHeads->True will place arrowheads on
the slope lines.";

ScaleArrows::usage = "ScaleArrows is an option for
DirectionField. It is set to 0.8 by default, but any positive
value is acceptable. Smaller values will produce shorter arrows.";

Begin["`Private`"] (* RMCPackages`ODETools`Private` *)

Options[DirectionField]={ArrowHeads->False,ScaleArrows->0.8,
	PlotPoints->11};

DirectionField[f_,{x_,x0_,x1_},{y_,y0_,y1_},opts___Rule]:=
  Module[{ah,sa,pp,sf,ar},
	{ah,sa,pp}={ArrowHeads,ScaleArrows,PlotPoints}/.{opts}/.Options[DirectionField];
	(* set ScaleFactor for nonoverlapping segments: *)
	sf=sa*Min[N[x1-x0],N[y1-y0]]/pp;
	(* set AspectRatio to Automatic only if x and y bounds allow: *)
	ar=If[(1/2<(x1-x0)/(y1-y0)&&(x1-x0)/(y1-y0)<2),Automatic,1/GoldenRatio];
	If[ah==False,
		PlotVectorField[{1,f},{x,x0,x1},{y,y0,y1}, 
			FilterOptions[PlotVectorField,opts]//Evaluate,
			Graphics`Common`GraphicsCommon`ScaleFunction->(1&),
			Graphics`Common`GraphicsCommon`ScaleFactor->sf,
			Graphics`Arrow`HeadLength->0,
			AspectRatio->ar,
			Axes->Automatic,
			PlotPoints->pp],
		PlotVectorField[{1,f},{x,x0,x1},{y,y0,y1}, 
			FilterOptions[PlotVectorField,opts]//Evaluate,
			Graphics`Common`GraphicsCommon`ScaleFunction->(1&),
			Graphics`Common`GraphicsCommon`ScaleFactor->sf,
			AspectRatio->ar,
			Axes->Automatic,
			PlotPoints->pp]
		]
	]

                
End[]   (* RMCPackages`ODETools`Private` *)

EndPackage[]    (* RMCPackages`ODETools` *)
