(* :Title: Polynomial Regression *)

(* :Context: RMCPackages`PolyRegression` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to fit a polynomial 
of arbitrary degree to a set of bivariate numerical data
using the method of least squares. Graphical displays of
the functions and residuals are available.
*)

(* :Package Version: 2.1 *)

(* :Mathematica Version: 3.0 *)

(* :History:
	V1.0 by Bruce F. Torrence, 1995.
	V2.0 by Bruce F. Torrence, 1998.
	V2.1 by Bruce F. Torrence, August 2000. 
*)

(* :Keywords: regression, bivariate, data analysis, polynomial *)


BeginPackage["RMCPackages`PolyRegression`"]


QuadraticFit::usage = "QuadraticFit[xdata, ydata] gives the equation
of the least-squares quadratic function for the data,  a scatterplot 
of the data with the quadratic function superimposed, and a residual 
plot. It will take any options that ListPlot takes. QuadraticFit[data],
where data is a list of coordinate pairs {{x1,y1},{x2,y2},...}, will 
give the same result." 

CubicFit::usage = "CubicFit[xdata, ydata] gives the equation
of the least-squares cubic function for the data,  a scatterplot 
of the data with the cubic function superimposed, and a residual 
plot. It will take any options that ListPlot takes. CubticFit[data],
where data is a list of coordinate pairs {{x1,y1},{x2,y2},...}, will 
give the same result."  

QuarticFit::usage = "QuarticFit[xdata, ydata] gives the equation
of the least-squares quartic function for the data,  a scatterplot 
of the data with the quartic function superimposed, and a residual 
plot. It will take any options that ListPlot takes. QuarticFit[data],
where data is a list of coordinate pairs {{x1,y1},{x2,y2},...}, will 
give the same result."   

PolynomialFit::usage = "PolynomialFit[xdata, ydata, n] gives the equation
of the least-squares degree n polynomial for the data,  a scatterplot 
of the data with the polynomial superimposed, and a residual 
plot. It will take any options that ListPlot takes. PolynomialFit[data,n],
where data is a list of coordinate pairs {{x1,y1},{x2,y2},...}, will 
give the same result."   


Begin["`Private`"]

PolynomialFit::badarg1 = "Oops, the first argument to `1` needs to be
either a list of bivariate data of the form {{x1,y1},{x2,y2},...}, 
or a pair of lists of the form {x1,x2,...},{y1,y2,...}.";

PolynomialFit::badarg2 = "Oops, your xdata list
is not the same length as your ydata list. Check your data entry
for a typo.";

PolynomialFit::badarg3 = "There are too few data points to warrent
fitting a polynmial of the requested degree. The polynomial below
has degree one less than the number of data points, and fits the data
perfectly.";

PolynomialFit::badarg4 = "Oops, the degree of a polynomial cannot
be negative.";

Lo[vec_List]:=1.1 Min[vec] - .1 Max[vec] 
(* a little smaller than the min *)

Hi[vec_List]:=1.1 Max[vec] - .1 Min[vec]
(* a little larger than the max *)

PerformPolynomialFit[xdata_List,ydata_List,
  n_Integer, opts___Rule]:=
 Module[{data,poly,i,l,p,pred,resid,r,rsegs},
  data=Transpose[{xdata,ydata}];
  poly=Fit[data,Table[x^i,{i,0,n}],x];       
  Print["y = ", poly /. x->"x"];
  l=ListPlot[data, opts,
		AxesOrigin->{Lo[xdata],Lo[ydata]}, 
        PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}},
		Prolog->AbsolutePointSize[3],
        DisplayFunction->Identity];
  p=Plot[poly, {x,Lo[xdata],Hi[xdata]},
        PlotStyle->{Hue[.7]},
        DisplayFunction->Identity];
  Show[l,p,DisplayFunction->$DisplayFunction,
        DefaultFont->{"Times",10},
        AxesLabel->{"x","y"}];
  pred=poly /. x->xdata;
  If[n==Length[xdata]-1,resid=ydata-ydata,
    resid=Chop[ydata-pred]];
  r=ListPlot[Transpose[{xdata,resid}], opts,
    AxesOrigin->{Lo[xdata], 0},
    PlotRange->{{Lo[xdata],Hi[xdata]},Automatic},
    Prolog->{AbsolutePointSize[3]},
    DisplayFunction->Identity
  ];
 rsegs=Insert[
          MapThread[Line[{{#1,#2},{#1,#3}}]&,
           {xdata,ydata-ydata,resid}],
	      Hue[1],1];     
  Show[r,Graphics[rsegs],DisplayFunction->$DisplayFunction,
        DefaultFont->{"Times",10},
        AxesLabel->{"x", "Residual"}]; 
 ]
 
PerformPolynomialFit[data_List, n_Integer, opts___Rule]:=
 Module[{tdata=Transpose[data]},
  PerformPolynomialFit[tdata[[1]],tdata[[2]],n,opts]
 ]
 

PolynomialFit[xdata_List,ydata_List, n_Integer, opts___Rule]:=
If[Length[xdata]!=Length[ydata],
    Message[PolynomialFit::badarg2],
  If[n<0,Message[PolynomialFit::badarg4],  
    If[n>Length[xdata]-1,
      Message[PolynomialFit::badarg3];
    PerformPolynomialFit[xdata,ydata,Length[xdata]-1,opts],
    PerformPolynomialFit[xdata,ydata,n,opts]
    ]
  ]
] 
 
PolynomialFit[data_List, n_Integer, opts___Rule]:=
If[Dimensions[data]!={Length[data],2},
  Message[PolynomialFit::badarg1, PolynomialFit],
  If[n<0,Message[PolynomialFit::badarg4],  
    If[n>Length[data]-1,
      Message[PolynomialFit::badarg3];
    PerformPolynomialFit[data,Length[data]-1,opts],
    PerformPolynomialFit[data,n,opts]
    ]
  ]
] 
 

QuadraticFit[xdata_List,ydata_List, opts___Rule]:=
 PolynomialFit[xdata,ydata,2,opts]

QuadraticFit[data_List, opts___Rule]:=
 If[Dimensions[data]!={Length[data],2},
  Message[PolynomialFit::badarg1, QuadraticFit],
  PolynomialFit[data,2,opts]
 ]

CubicFit[xdata_List,ydata_List, opts___Rule]:=
 PolynomialFit[xdata,ydata,3,opts]

CubicFit[data_List, opts___Rule]:= 
  If[Dimensions[data]!={Length[data],2},
  Message[PolynomialFit::badarg1, CubicFit],
  PolynomialFit[data,3,opts]
 ]

QuarticFit[xdata_List,ydata_List, opts___Rule]:=
 PolynomialFit[xdata,ydata,4,opts]
 
QuarticFit[data_List, opts___Rule]:= 
  If[Dimensions[data]!={Length[data],2},
  Message[PolynomialFit::badarg1, QuarticFit],
  PolynomialFit[data,4,opts]
 ]

End[]

EndPackage[]