(* :Title: Sequence Tools *)

(* :Context: RMCPackages`SequenceTools` *)

(* :Author: Bruce F. Torrence *)

(* :Summary: 
This package allows the user to easily plot and 
make tables of sequences. Tools are also provided
for producing difference tables for sequences. 
*)

(* :Package Version: 2.1 *)

(* :Mathematica Version: 4.1 *)

(* :History:
	V1.0 by Bruce F. Torrence, December 1997.
	V2.0 by Bruce F. Torrence, August 2000. 
	V2.1 by Bruce F. Torrence, January 2002.
*)

(* :Keywords: sequence, difference equation, logistic growth, cobweb, 
	bifurcation *)


BeginPackage["RMCPackages`SequenceTools`", "Statistics`LinearRegression`", 
	"Utilities`FilterOptions`"]

BifurcationPlot::usage="BifurcationPlot[f, x0, {a, amin, amax}] produces a
bifurcation diagram for a function f[x] which is defined in terms of the 
parameter a. (A good example function to start with is f[x_]:= a*x*E^-x, 
with amin=2 and amax=20, and x0 between them). For each of many sampled values 
of the parameter a between amin and amax, the plot shows the long-term orbit of 
x0 under repeated iteration of f. By default, above each a-value on the horizontal 
axis appears the values Nest[f,x0,100] through Nest[f,x0,200]. This can be adjusted 
with the option Iterations, whose default setting is Iterations->{100,200}.
By default, the interval from amin to amax is subdivided into 99 subintervals of equal 
width, and the 100 endpoints of the subintervals are sampled. This can be adjusted
in either of two ways: the first is to use the option Subintervals, whose default 
setting is 99. The second is to use the syntax BifurcationPlot[f, x0, {a, amin, amax, da}],
where da is the width of a single subinterval (the distance between successive sample 
points).For the true control freaks, the option Subintervals may also be set to a
list of values that are to be sampled. BifurcationPlot will accept any options
accepted by ListPlot.
A different use of BifurcationPlot is also possible. What if the parameter a is 
fixed at the value a0 and the intitial value x0 is permitted to vary? Simply adjust 
the syntax to reflect this change:
BifurcationPlot[f, {x0min, x0max}, {a, a0}] does the job. In this case the horizontal 
axis represents the varying values of the initial point x0. The options Iterations
and Subintervals work as above. One can also proceed in this investigation for a
function f defined without a parameter (such as f[x]=2*x*E^-x). In this case, use the
even simpler syntax BifurcationPlot[f, {x0min, x0max}] to get the plot."

Cobweb::usage="Cobweb[f, x0, n] gives a visual representation of the 
sequence {x0, f[x0], f[f[x0]], ...} where each term is obtained from
the previous term by applying the function f, and where the last term
is obtained by applying f precisely n times. The command requires that
f be a real-valued function of a single variable, x0 be a real number,
and n be a nonnegative integer. The sequence itself can
be generated with the command NestList[f, x0, n]. The Cobweb command
produces a plot of f togther with the line y=x, and the `cobweb'
comprised of the horizontal and vertical segments between these graphs.
Also displayed are red dots on the x axis indicating the values of the
sequence produced. Cobweb will accept any option accepted by the Plot
command; in particular, the setting AspectRatio->Automatic can be used
to give both axes the same scale. Also, the PlotRange option can be
used to adjust the range of values on each axis."

CobwebMovie::usage="CobwebMovie[f, x0, n] gives a visual representation of the 
sequence {x0, f[x0], f[f[x0]], ...} where each element is obtained from
the previous element by applying the function f, and where the last member
is obtained by applying f precisely n times. The command requires that
f be a real-valued function of a single variable, x0 be a real number,
and n be a nonnegative integer. The sequence itself can
be generated with the command NestList[f, x0, n]. The CobwebMovie command
produces a movie, that is a sequence of graphics frames. Each frame shows a 
plot of f togther with the line y = x, and the emerging `cobweb'
comprised of the horizontal and vertical segments between these graphs.
Also displayed are dots on the x axis indicating the values of the
emerging sequence. CobwebMovie will accept any option accepted by the Plot
command; in particular, the setting AspectRatio->Automatic can be used
to give both axes the same scale, and the PlotRange option can be
used to adjust the range of values on each axis. There is one frame in the
movie for each of the n+1 members of the sequence."

Diff::usage = "Diff[y] gives the list of differences between successive
members of the list y. The n-th member of Diff[y] is y[[n+1]]-y[[n]]."

\[CapitalDelta]::usage = "\[CapitalDelta][y, n] gives the n-th differences
for the list of values y. \[CapitalDelta][y, 1] is the same as Diff[y], 
\[CapitalDelta][y, 2] is the same as Diff[Diff[y]], etc."

DeltaTable::usage = "DeltaTable[y, n] produces a table of first through
n-th differences for the values of the sequence y. To use the command, the 
first argument y must be a list of the first several values of a sequence.
If the values are empirical data, be sure that the corresponding x-values
are evenly spaced (i.e. the first differencs of the x data should be constant)." 

LogisticFit::usage = "LogisticFit[xdata,ydata] gives the coefficients for
the `best fitting' logistic growth sequence for the list of values
ydata. The best fit sequence is given by the difference equation
p[n]=a*p[n-1]-b*p[n-1]^2, with initial value corresponding to the first
value in ydata. LogisticFit[data], where data is a list of
two-tuples {{x1,y1},{x2,y2},...}, will give the same result.
LogisticFit assumes the members of xdata are
evenly spaced, and will give poor results when this is not the case.
The output also includes a scatterplot of the original data, with the 
best fitting sequence superimposed in blue."

SequencePlot::usage = "SequencePlot[s[n], {n, nmin, nmax}] produces a 
plot of the sequence s, as n ranges from nmin to nmax. 
SequencePlot[s[n], {n, nmin, nmax, step}] produces a 
plot of the sequence s, as n ranges from nmin to nmax in increments of step. 
SequencePlot will accept any option that ListPlot does. In particular, 
you can connect the dots by adding the option PlotJoined->True. SequencePlot
will also accept a list of sequences as its first argument."

SequenceTable::usage = "SequenceTable[s, {n, nmin, nmax}] produces a
table of values for the sequence s as n ranges from nmin to nmax.
SequenceTable[s, {n, nmin, nmax, step}] produces a table of values
where n ranges from nmin to nmax in increments of step. SequenceTable
will accept any options that TableForm does. SequenceTable
will also accept a list of sequences as its first argument."

BifurcationPlot::badarg1=
    "Oops! Your third argument must be a list {a, amin, amax} specifying the name 
    of the parameter and the numerical values it is to range between, or a list 
    {a, amin, amax, da} specifying the the same information and the increment 
    between successive 'a' values. In either case, amin and amax must be real numbers.";

BifurcationPlot::badarg2=
    "Oops! Your second argument must be a list {x0min, x0max} specifying the 
    domain for your plot, or a list {x0min, x0max, dx} specifying the domain for 
    your plot and the increment between successive initial values. In either case, 
    x0min and x0max must be real numbers.";

BifurcationPlot::badarg3=
    "Something is wrong with your input. The second argument should be of the form 
    {xmin, xmax, dx} where dx is a small positive number, or it may be of the form 
    {xmin, xmax} with an option setting such as Subintervals->100.";

BifurcationPlot::badarg4=
    "The option Iterations should be set to a list of the form {Start value, End value}. 
    The bifurcation plot will show for each initial value x the orbit of x, starting 
    with the start value number of iterations and ending with the end value number of 
    iterations. The defualt setting is Iterations->{100,200}.";

BifurcationPlot::badarg5=
    "The option Iterations should be set to a list of the form {Start value, End value}, 
    and the end value MUST be greater than the start value.";

BifurcationPlot::badarg6=
    "Oops! Your second argument must be a real number. It represents the initial value 
    x0 over which f is to be iterated.";

BifurcationPlot::badarg7=
    "Oops! Your third argument must be of the form {a, a0}, where a is the name of the 
    parameter used in the definition of the first argument f, and a0 is the numerical 
    value that a is to assume. a0 must be a real number.";

Cobweb::badarg2 = "Oops, your second argument `1` needs to be
a real number. It is the initial value.";

Cobweb::badarg3 = "Oops, your third argument `1` needs to be
a nonnegative integer. It is the number of iterations to be performed.";

Cobweb::diverge = "Sorry. The sequence x0, f[x0], f[f[x0]], ... blew up. The largest
number exceeded the maximum machine-size number for this computer, and hence could
not be plotted (it would not be very interesting anyway). Try a smaller number of iterations,
or a different initial value.";

Options[BifurcationPlot]={
	Subintervals->99,
    Iterations->{100,200}
    };


Begin["`Private`"]

(****** Begin BifurcationPlot definitions *)

subdivide[{a_,b_},n_]:=Module[{dx=(N[b]-a)/n,i},Table[a+i*dx,{i,0,n}]]

BifurcationPlot[
    f_,{xmin_?NumericQ,xmax_?NumericQ,delx_:Automatic},{a_, a0_?NumericQ}, 
    opts___Rule]:=
  Module[{dx=N[delx],subint,iter,xvals,i,start,end,x,newpts,ptlst},
    {subint,iter}={Subintervals,Iterations}/.{opts}/.Options[BifurcationPlot];
    If[Head[iter]=!=List,Message[BifurcationPlot::badarg4];Return[]];
    start=iter[[1]];
    end=iter[[2]]-start;
    If[end<0,Message[BifurcationPlot::badarg5];Return[]];
    xvals=Which[
        NumericQ[dx]&&subint==Subintervals/.Options[BifurcationPlot],
        Union[N[Table[i,{i,xmin,xmax,dx}]],{N[xmax]}],
        NumericQ[dx]&&IntegerQ[subint]&&subint>=1,
        Union[N[Table[xmin+i*dx,{i,0,subint-1}]],{N[xmax]}],
        dx==Automatic&&IntegerQ[subint]&&subint>=1,
        subdivide[{N[xmin],xmax},subint],
        Head[subint]===List,N[subint],
        True,Message[BifurcationPlot::badarg3];Return[]];
    ptlst={};
    Do[x=xvals[[n]];
      newpts=
        Union[{x,#}&/@
            NestList[(f[#]/.a\[Rule]a0)&,Nest[(f[#]/.a\[Rule]a0)&,x,start],end]];
      ptlst=Join[ptlst,newpts],
      {n,Length[xvals]}];
    Print["Parameter ",a," fixed at ",a0,
      " while initial value x0 varies from ",xmin," to ",xmax,"."];
    ListPlot[ptlst,Evaluate[FilterOptions[ListPlot,opts]],
      AxesLabel->{"Initial Value","Long-term Orbit"}]
    ]
    
BifurcationPlot[f_,{xmin_,xmax_,delx_:Automatic},{a_,a0_?NumericQ},opts___Rule]:=
  Message[BifurcationPlot::badarg2]
  
BifurcationPlot[f_,{xmin_?NumericQ,xmax_?NumericQ,delx_:Automatic},{a_,a0_},
    opts___Rule]:=
  Message[BifurcationPlot::badarg7]  

BifurcationPlot[f_,{xmin_?NumericQ,xmax_?NumericQ,delx_:Automatic},opts___Rule]:=
  Module[{dx=N[delx],subint,iter,xvals,i,start,end,x,newpts,ptlst},
    {subint,iter}={Subintervals,Iterations}/.{opts}/.Options[BifurcationPlot];
    If[Head[iter]=!=List,Message[BifurcationPlot::badarg4];Return[]];
    start=iter[[1]];
    end=iter[[2]]-start;
    If[end<0,Message[BifurcationPlot::badarg5];Return[]];
    xvals=Which[
        NumericQ[dx]&&subint==Subintervals/.Options[BifurcationPlot],
        Union[N[Table[i,{i,xmin,xmax,dx}]],{N[xmax]}],
        NumericQ[dx]&&IntegerQ[subint]&&subint>=1,
        Union[N[Table[xmin+i*dx,{i,0,subint-1}]],{N[xmax]}],
        dx==Automatic&&IntegerQ[subint]&&subint>=1,
        subdivide[{N[xmin],xmax},subint],
        Head[subint]===List,N[subint],
        True,Message[BifurcationPlot::badarg3];Return[]];
    ptlst={};
    Do[x=xvals[[n]];
      newpts=Union[{x,#}&/@NestList[f,Nest[f,x,start],end]];
      ptlst=Join[ptlst,newpts],
      {n,Length[xvals]}];
    Print["Initial value x0 varies from ",xmin," to ",xmax,"."];
    ListPlot[ptlst,Evaluate[FilterOptions[ListPlot,opts]],
      AxesLabel->{"Initial Value","Long-term Orbit"}]
    ]

BifurcationPlot[f_,{xmin_,xmax_,delx_:Automatic},opts___Rule]:=
  Message[BifurcationPlot::badarg2]

BifurcationPlot[f_,
    x0_?NumericQ,{a_,amin_?NumericQ,amax_?NumericQ,dela_:Automatic},opts___Rule]:=
  Module[{da=N[dela],subint,iter,avals,i,start,end,aa,newpts,ptlst},
    {subint,iter}={Subintervals,Iterations}/.{opts}/.Options[BifurcationPlot];
    If[Head[iter]=!=List,Message[BifurcationPlot::badarg4];Return[]];
    start=iter[[1]];
    end=iter[[2]]-start;
    If[end<0,Message[BifurcationPlot::badarg5];Return[]];
    avals=
      Which[NumericQ[da]&&subint==Subintervals/.Options[
            BifurcationPlot],Union[N[Table[i,{i,amin,amax,da}]],{N[amax]}],
        NumericQ[da]&&IntegerQ[subint]&&subint>=1,
        Union[N[Table[amin+i*da,{i,0,subint-1}]],{N[amax]}],
        da==Automatic&&IntegerQ[subint]&&subint>=1,
        subdivide[{N[amin],amax},subint],
        Head[subint]===List,N[subint],
        True,Message[BifurcationPlot::badarg3];Return[]];
    ptlst={};
    Do[aa=avals[[n]];
      newpts=
        Union[{aa,#}&/@
            NestList[(f[#]/.a\[Rule]aa)&,Nest[(f[#]/.a\[Rule]aa)&,x0,start],end]];
      ptlst=Join[ptlst,newpts],
      {n,Length[avals]}];
    Print["Initial value x0 fixed at ",x0," while parameter ",a,
      " varies from ",amin," to ",amax,"."];
    ListPlot[ptlst,Evaluate[FilterOptions[ListPlot,opts]],
      AxesLabel->{"Value of Parameter","Long-term Orbit"}]
    ]

BifurcationPlot[f_,x0_,{a_,amin_?NumericQ,amax_?NumericQ,dela_:Automatic},
    opts___Rule]:=
  Message[BifurcationPlot::badarg6]

BifurcationPlot[f_,x0_?NumericQ,{a_,amin_,amax_,dela_:Automatic},opts___Rule]:=
  Message[BifurcationPlot::badarg1]


(****** End BifurcationPlot definitions *)
(****** Begin Cobweb and CobwebMovie definitions *)


Cobweb[f_,x0_?NumericQ,n_Integer?NonNegative, opts___Rule]:=Module[{},
    li=Flatten[NestList[Map[f,#,{2}]&,{{x0,x0},{x0,f[x0]}},n],1];
    li[[1]]={x0,0};
    weblines=Graphics[{Hue[.7],Line[li]}];
    xlist=NestList[f,x0,n];
    xmin=Min[xlist];
    xmax=Max[xlist];
    If[Max[Abs[xmin],Abs[xmax]]>$MaxMachineNumber,Return[Message[Cobweb::diverge]]];
    pts=Graphics[{Hue[1],PointSize[.015],Point[{#,0}]&/@xlist}];
    If[xmin\[Equal]xmax ,xmin=xmin-1;xmax=xmax+1];
    xrange=Max[xmax-xmin,2*x0];
    plotf=
      Plot[{f[x],x},{x,Min[0,xmin-.05 xrange],xmax+.2xrange},opts,
        DisplayFunction\[Rule]Identity];
    Show[plotf,weblines,pts,DisplayFunction\[Rule]$DisplayFunction];
    ];
    
Cobweb[f_, x0_, n_Integer?NonNegative, opts___Rule]:=
  Message[Cobweb::badarg2, x0];    

Cobweb[f_, x0_?NumericQ, n_, opts___Rule]:=
  Message[Cobweb::badarg3, n]; 
  
CobwebMovie[f_,x0_?NumericQ,n_Integer?Positive,opts___Rule]:=
    Module[{x0n=N[x0],li,xlist,pts,newpt,oldpts,weblines,xmin,xmax,xrange,
        plotf},
      li=Flatten[NestList[Map[f,#,{2}]&,{{x0n,x0n},{x0n,f[x0n]}},n],1];
      li[[1]]={x0n,0};
      xlist=NestList[f,x0n,n];
      pts=Point[{#,0}]&/@xlist;
      newpt[k_]:=Graphics[{Hue[1],PointSize[.015],pts[[k]]}];
      oldpts[k_]:=
        Graphics[{RGBColor[0.64,0.25,0.34],PointSize[.015],Take[pts,k-1]}];
      weblines[k_]:=Graphics[{Hue[.7],Line[Take[li,2k+1]]}];
      xmin=Min[xlist];
      xmax=Max[xlist];
      If[xmin==xmax,xmin=xmin-1;xmax=xmax+1];
      xrange=Max[xmax-xmin,2*x0n];
      plotf=
        Plot[{f[x],x},{x,Min[0,xmin-.05 xrange],xmax+.2xrange},opts,
          DisplayFunction->Identity];
      Show[plotf,newpt[1],DisplayFunction->$DisplayFunction];
      Do[
        Show[plotf,weblines[k],oldpts[k+1],newpt[k+1],
          DisplayFunction->$DisplayFunction]
        ,{k,n}];
      ]
      
CobwebMovie[f_,x0_,n_Integer?Positive,opts___Rule]:=
    Message[CobwebMovie::badarg2,x0]
    
CobwebMovie[f_,x0_?NumericQ,n_,opts___Rule]:=
    Message[CobwebMovie::badarg3,n]        
  
(***** End Cobweb and CobwebMovie Definitions *) 
(***** Begin Remaining command definitions *)    

Diff[y_List]:= Drop[y,1] - Drop[y,-1]
(* the first difference, to be used iteratively later *)

\[CapitalDelta][y_List,n_Integer]:=If[n==0, y, Nest[Diff, y, n]]
(* the n-th difference *)

Star[n_]:=Table["*",{i,n}]

Col[y_List,n_]:=Join[\[CapitalDelta][y,n] , Star[n]]
(* the column of the table with n-th differences. The stars added at
the end to make each column the same length *)
 
T[y_List, n_]:=Transpose[Table[Col[y,i],{i,0,n}]]

DeltaTable[y_List,n_]:=
	TableForm[T[y,n],
		TableHeadings->{{}, Join[{"y"},Table["\[CapitalDelta]"^i,{i,n}]]}]
 
Lo[vec_List]:=1.1 Min[vec] - .1 Max[vec] 
(* a little smaller than the min - used for plot ranges *)

Hi[vec_List]:=1.1 Max[vec] - .1 Min[vec]
(* a little larger than the max - used for plot ranges *)

LogisticFit[xdata_List,ydata_List, opts___Rule]:=
 Module[{data,regdata,reg,line,aa,bb,l,s,preddata,predplot},
  data=Transpose[{xdata,ydata}];
  regdata=Transpose[{Drop[ydata,-1] , Diff[ydata]/Drop[ydata,-1]}];
  reg=Regress[regdata,{1,x},x,
      RegressionReport->{RSquared}];
  line=Fit[regdata,{1,x},x];
  aa=line /. x->0;
  bb=line - aa /. x->-1;       
  Print["a = ", aa];
  Print["b = ", bb];
  Print[r^2 /. r->"r"," = ", 
		Statistics`Common`RegressionCommon`RSquared /. reg];
  l=ListPlot[data, opts,
		AxesOrigin->{Lo[xdata],Lo[ydata]}, 
        PlotRange->{{Lo[xdata],Hi[xdata]},{Lo[ydata],Hi[ydata]}},
		Prolog->AbsolutePointSize[3],
        DisplayFunction->Identity];
  s[1]=ydata[[1]];
  s[n_]:=s[n]=(1+aa) s[n-1] - bb s[n-1]^2;
  preddata=Transpose[{xdata, Table[s[n],{n,Length[xdata]}] }];      
  predplot=ListPlot[preddata, PlotJoined->True,
        PlotStyle->{Hue[.7]}, 
        DisplayFunction->Identity];
  Show[l,predplot,DisplayFunction->$DisplayFunction]
 ]

 LogisticFit[data_List, opts___Rule]:=
  Module[{tdata=Transpose[data]},
   LogisticFit[tdata[[1]],tdata[[2]],opts]
 ]
 
 SequencePlot[funclist_List, {n_,nmin_Integer,nmax_Integer,step___Integer}, 
    opts___Rule]:=
  Module[{datapoints,xpoints,ypoints,fypoints,axesorigin,plotrange,m,huelist,
      colordatapoints,plots} ,
	datapoints=Map[Table[{n,#},{n,nmin,nmax,step}]&,funclist];
	xpoints=Range[nmin,nmax,step];
	ypoints=Map[Table[#,{n,nmin,nmax,step}]&,funclist];
	fypoints=Flatten[ypoints];
	axesorigin={Max[Lo[xpoints],0],Lo[fypoints]};
	plotrange={{Max[Lo[xpoints],0],Hi[xpoints]},{Lo[fypoints],Hi[fypoints]}};
	m=Length[funclist];
	huelist=Which[m==1,{Hue[.7]},
				m==2,{Hue[.7],Hue[1]},
				m==3,{Hue[.7],Hue[1],Hue[.3]},
				m>3,Table[Hue[i/m],{i,m}]];
	colordatapoints=Thread[List[datapoints,huelist]];
	plots=Map[
        ListPlot[#[[1]],opts,PlotStyle->#[[2]],AxesOrigin->axesorigin,
            PlotRange->plotrange,Prolog->AbsolutePointSize[3],
            DisplayFunction->Identity]&,colordatapoints];
    Show[plots,DisplayFunction->$DisplayFunction]
  ]

 
 SequencePlot[func_,range_List, opts___Rule]:=
   SequencePlot[{func},range,opts]
 
 SequenceTable[funclist_List, {n_,nmin_Integer,nmax_Integer,step___Integer}, 
    opts___Rule]:=
  Module[{data} ,
   data=Table[Join[{n},funclist],{n,nmin,nmax,step}];
   TableForm[data,opts,
		TableHeadings->{None,Join[{n},funclist]}
		]	
   ]

 SequenceTable[s_, range_List, opts___Rule]:=
  SequenceTable[{s},range,opts]


End[]

EndPackage[]